package cn.wzpmc.commerce.config;

import cn.wzpmc.commerce.controllers.AuthController;
import cn.wzpmc.commerce.utils.MessageResponse;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.access.AccessDeniedHandler;

import java.io.IOException;

@Configuration
@EnableWebSecurity
public class SecurityConfig {
    @SneakyThrows
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity security) {
        security.authorizeHttpRequests(registry -> registry.requestMatchers("/api/auth/**",
                        "/swagger-ui/**",
                        "/swagger-ui.html",
                        "/v3/**").permitAll()
                .anyRequest().authenticated());
        security.csrf(AbstractHttpConfigurer::disable);
        security.cors(AbstractHttpConfigurer::disable);
        security.formLogin(httpSecurityFormLoginConfigurer -> httpSecurityFormLoginConfigurer
                .loginPage("/api/auth/notLogin")
                .loginProcessingUrl("/api/auth/login")
                .successHandler(AuthController::loginSuccess)
                .failureHandler(AuthController::loginFailed));
        security.logout(httpSecurityLogoutConfigurer -> httpSecurityLogoutConfigurer.logoutUrl("/api/auth/logout")
                .logoutSuccessHandler(AuthController::logoutSuccess)
                .permitAll());
        security.exceptionHandling(httpSecurityExceptionHandlingConfigurer -> httpSecurityExceptionHandlingConfigurer.authenticationEntryPoint(new MyAuthenticationEntryPoint())
                .accessDeniedHandler(new MyAccessDeniedHandler()));
        return security.build();
    }

    @Bean
    public BCryptPasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    private static class MyAuthenticationEntryPoint implements AuthenticationEntryPoint {

        @Override
        public void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException {
            String localizedMessage = authException.getLocalizedMessage();
            response.setStatus(403);
            response.setContentType("application/json");
            ServletOutputStream outputStream = response.getOutputStream();
            new MessageResponse(localizedMessage, 403).copyToStream(outputStream);
            outputStream.close();
        }
    }

    private static class MyAccessDeniedHandler implements AccessDeniedHandler {

        @Override
        public void handle(HttpServletRequest request, HttpServletResponse response, AccessDeniedException accessDeniedException) throws IOException {
            String localizedMessage = accessDeniedException.getLocalizedMessage();
            response.setStatus(404);
            response.setContentType("application/json");
            ServletOutputStream outputStream = response.getOutputStream();
            new MessageResponse(localizedMessage, 403).copyToStream(outputStream);
            outputStream.close();
        }
    }
}
