package cn.wzpmc.commerce.controllers;

import cn.wzpmc.commerce.entities.User;
import cn.wzpmc.commerce.entities.vo.AccountVo;
import cn.wzpmc.commerce.services.UserService;
import cn.wzpmc.commerce.utils.DataResponse;
import cn.wzpmc.commerce.utils.MessageResponse;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.web.bind.annotation.*;

@Tag(name = "AuthController", description = "用户权限、登录、注册控制器")
@RestController
@RequestMapping("/api/auth")
public class AuthController {
    private final UserService service;

    @Autowired
    public AuthController(UserService service) {
        this.service = service;
    }

    @SneakyThrows
    public static void loginSuccess(HttpServletRequest ignoredRequest, HttpServletResponse response, Authentication authentication) {
        response.setStatus(200);
        response.setContentType("application/json");
        ServletOutputStream outputStream = response.getOutputStream();
        new DataResponse<>((User) authentication.getPrincipal(), "登陆成功！").copyToStream(outputStream);
        outputStream.close();
    }

    @SneakyThrows
    public static void loginFailed(HttpServletRequest ignoredRequest, HttpServletResponse response, AuthenticationException authenticationException) {
        response.setStatus(403);
        response.setContentType("application/json");
        ServletOutputStream outputStream = response.getOutputStream();
        new MessageResponse(authenticationException.getLocalizedMessage(), 403).copyToStream(outputStream);
        outputStream.close();
    }

    @SneakyThrows
    public static void logoutSuccess(HttpServletRequest ignoredRequest, HttpServletResponse response, Authentication ignoredAuthentication) {
        response.setStatus(200);
        response.setContentType("application/json");
        ServletOutputStream outputStream = response.getOutputStream();
        new MessageResponse("登出成功！").copyToStream(outputStream);
        outputStream.close();
    }

    @SneakyThrows
    @Hidden
    @GetMapping("/notLogin")
    public void notLogin(HttpServletResponse response) {
        response.setStatus(403);
        response.setContentType("application/json");
        ServletOutputStream outputStream = response.getOutputStream();
        new MessageResponse("未登录", 403).copyToStream(outputStream);
        outputStream.close();
    }

    @SneakyThrows
    @Operation(summary = "注册", description = "注册一个账号")
    @ApiResponse(responseCode = "200", description = "返回一个用户对象")
    @PostMapping("/register")
    public MessageResponse register(@RequestBody AccountVo accountVo) {
        User user = service.addUser(accountVo);
        if (user != null) {
            return new DataResponse<>(user, "注册成功！");
        }
        return new MessageResponse("注册失败！用户已存在！", 304);
    }
}
