package cn.wzpmc.commerce.controllers;

import cn.wzpmc.commerce.entities.User;
import cn.wzpmc.commerce.entities.vo.PermissionVo;
import cn.wzpmc.commerce.services.PermissionService;
import cn.wzpmc.commerce.utils.DataResponse;
import cn.wzpmc.commerce.utils.MessageResponse;
import cn.wzpmc.commerce.utils.PageableResponse;
import cn.wzpmc.commerce.utils.ServiceUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@Tag(name = "权限控制器", description = "权限管理器（CRUD）")
@RequestMapping("/authPermissions")
public class PermissionController {
    private final PermissionService permissionService;

    @Autowired
    public PermissionController(PermissionService permissionService) {
        this.permissionService = permissionService;
    }

    @Operation(summary = "添加一个权限")
    @PostMapping(path = "/addAuthPermissions")
    public MessageResponse addAuthPermissions(@RequestBody PermissionVo authPermissionVo, Authentication authentication) {
        authPermissionVo.setCreateId(((User) authentication.getPrincipal()).getId());
        if (permissionService.save(authPermissionVo)) {
            return new MessageResponse("添加成功！");
        }
        return new MessageResponse("添加失败！", 500);
    }

    @Operation(summary = "修改权限")
    @PostMapping(path = "/changeAuthPermissions")
    public MessageResponse changeAuthPermissions(@RequestBody PermissionVo authPermissionVo) {
        if (ServiceUtils.update(permissionService, authPermissionVo)) {
            return new MessageResponse("修改成功！");
        }
        return new MessageResponse("修改失败！", 500);
    }

    @Operation(summary = "删除权限")
    @GetMapping(path = "/delAuthPermissions")
    public MessageResponse delAuthPermissions(@RequestParam("id") int id) {
        if (permissionService.removeById(id)) {
            return new MessageResponse("删除成功！");
        }
        return new MessageResponse("删除失败，未找到！", 500);
    }

    @Operation(summary = "分页获取权限")
    @GetMapping(path = "/getAuthPermissions")
    public PageableResponse<PermissionVo> getAuthPermissions(@RequestParam("page") int page, @RequestParam("num") int num) {
        return PageableResponse.of(permissionService.page(new Page<>(page, num)));
    }

    @Operation(summary = "修改一个权限的状态")
    @GetMapping(path = "/changeAuthPermissionsStats")
    public MessageResponse changeAuthPermissionsStats(@RequestParam("authPermissionId") int id, @RequestParam("status") boolean status) {
        if (ServiceUtils.updateStatus(this.permissionService, id, status)) {
            return new MessageResponse("修改成功！");
        }
        return new MessageResponse("修改失败！", 500);
    }

    @Operation(summary = "获取所有权限")
    @GetMapping(path = "/getAuthPermissionsId")
    public DataResponse<List<PermissionVo>> getAuthPermissionsId() {
        return new DataResponse<>(permissionService.list());
    }
}
