package cn.wzpmc.commerce.controllers;

import cn.wzpmc.commerce.entities.User;
import cn.wzpmc.commerce.entities.vo.AccountVo;
import cn.wzpmc.commerce.services.UserService;
import cn.wzpmc.commerce.utils.DataResponse;
import cn.wzpmc.commerce.utils.MessageResponse;
import cn.wzpmc.commerce.utils.PageableResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

@Tag(name = "用户账号控制器", description = "用户的增删改查")
@RestController
@RequestMapping("/account")
public class UserController {
    private final UserService userService;

    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @Operation(summary = "更改账户信息", description = "使用传入账号的id对账户信息进行更改（包括用户名、邮箱、真名、权限组）")
    @PostMapping("/changeAccount")
    public MessageResponse changeAccount(@RequestBody AccountVo accountVo) {
        return this.userService.changeAccount(accountVo);
    }

    @Operation(summary = "修改密码", parameters = {@Parameter(name = "userId", description = "用户id"), @Parameter(name = "oldPassword", description = "旧密码"), @Parameter(name = "newPassword", description = "新密码")})
    @PostMapping("/changePassword")
    public MessageResponse changePassword(@RequestParam("userId") Integer userId, @RequestParam("oldPassword") String oldPassword, @RequestParam("newPassword") String newPassword) {
        return this.userService.changePassword(userId, oldPassword, newPassword);
    }

    @Operation(summary = "删除用户", parameters = {@Parameter(name = "id", description = "用户id")})
    @GetMapping("/delUser")
    public MessageResponse delUser(@RequestParam("id") Integer id, Authentication authentication) {
        if (id.equals(((User) authentication.getPrincipal()).getId())) {
            return new MessageResponse("不能删除自己！", 403);
        }
        return this.userService.delUser(id);
    }

    @Operation(summary = "获取用户", parameters = {@Parameter(name = "page", description = "页数"), @Parameter(name = "num", description = "每页数量")})
    @GetMapping("/getUser")
    public PageableResponse<AccountVo> getUser(@RequestParam("page") Long page, @RequestParam("num") Long num, Authentication authentication) {
        return this.userService.getUser(page, num, authentication);
    }

    @Operation(summary = "获取当前用户信息")
    @GetMapping("/getUserInfo")
    public DataResponse<User> getUserInfo(Authentication authentication) {
        return new DataResponse<>((User) authentication.getPrincipal());
    }
}
