package cn.wzpmc.commerce.utils;

import cn.wzpmc.commerce.entities.User;
import lombok.NoArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.core.context.SecurityContextHolder;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class EntitiesUtils {

    @SneakyThrows
    public static <F, T> T copyField(F from, T to, NameParser parser) {
        Class<?> fromClass = from.getClass();
        Class<?> toClass = to.getClass();
        Method[] fromMethods = fromClass.getDeclaredMethods();
        Method[] toMethods = toClass.getDeclaredMethods();
        Map<String, Method> getters = new HashMap<>();
        for (Method fromMethod : fromMethods) {
            String methodName = fromMethod.getName();
            if ((methodName.startsWith("get") || methodName.startsWith("is")) && fromMethod.getParameters().length == 0) {
                getters.put(parser.getterParse(methodName), fromMethod);
            }
        }
        for (Method toMethod : toMethods) {
            String methodName = toMethod.getName();
            Parameter[] parameters = toMethod.getParameters();
            if (methodName.startsWith("set") && parameters.length == 1) {
                String fieldName = parser.setterParse(methodName);
                if (getters.containsKey(fieldName)) {
                    Method getter = getters.get(fieldName);
                    Class<?> returnType = getter.getReturnType();
                    if (!parameters[0].getType().equals(returnType)) {
                        continue;
                    }
                    Object fieldValue = getter.invoke(from);
                    toMethod.invoke(to, fieldValue);
                }
            }
        }
        return to;
    }

    public static <F, T> T copyField(F from, T to) {
        return copyField(from, to, new NameParser());
    }

    @SneakyThrows
    public static <T> void addCreateId(T obj) {
        Class<?> aClass = obj.getClass();
        int userId = ((User) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getId();
        for (Field declaredField : aClass.getDeclaredFields()) {
            declaredField.setAccessible(true);
            if (declaredField.getName().equals("create_id")) {
                declaredField.set(obj, userId);
                return;
            }
        }
        log.warn("在类{}中未找到create_id", aClass);
    }

    @NoArgsConstructor
    public static class NameParser {
        private final HashMap<String, String> getterParsers = new HashMap<>();
        private final HashMap<String, String> setterParsers = new HashMap<>();

        public NameParser addParser(String from, String to) {
            this.getterParsers.put(from, to);
            this.setterParsers.put(from, to);
            return this;
        }

        public NameParser addGetter(String from, String to) {
            this.getterParsers.put(from, to);
            return this;
        }

        public NameParser addSetter(String from, String to) {
            this.setterParsers.put(from, to);
            return this;
        }

        private String getterParse(String from) {
            if (from.startsWith("get")) {
                from = from.replaceFirst("get", "");
            } else {
                from = from.replaceFirst("is", "");
            }
            return getterParsers.getOrDefault(from, from);
        }

        private String setterParse(String from) {
            from = from.replaceFirst("set", "");
            return setterParsers.getOrDefault(from, from);
        }
    }
}
