package cn.wzpmc.commerce.utils;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;
import java.util.Objects;

@Slf4j
public class ServiceUtils {
    @SneakyThrows
    public static <T> boolean update(IService<T> service, T entities) {
        Class<T> entityClass = service.getEntityClass();
        Field[] declaredFields = entityClass.getDeclaredFields();
        UpdateWrapper<T> updateWrapper = new UpdateWrapper<>();
        Object id = null;
        String idName = "id";
        for (Field field : declaredFields) {
            field.setAccessible(true);
            String fieldName = field.getName();
            TableId tableId = field.getDeclaredAnnotation(TableId.class);
            if (Objects.nonNull(tableId) || fieldName.equals("id")) {
                id = field.get(entities);
                if (Objects.nonNull(tableId)) {
                    idName = tableId.value();
                }
            }
            Updatable declaredAnnotation = field.getDeclaredAnnotation(Updatable.class);
            if (declaredAnnotation != null) {
                TableField tableField = field.getDeclaredAnnotation(TableField.class);
                String name;
                if (tableField == null) {
                    name = fieldName;
                } else {
                    name = tableField.value();
                }
                Object value = field.get(entities);
                updateWrapper.set(Objects.nonNull(value), name, value);
            }
        }
        if (id == null) {
            log.warn("类{}未找到id", entityClass);
            return false;
        }
        updateWrapper.eq(idName, id);
        return service.update(updateWrapper);
    }

    public static <T> boolean updateStatus(IService<T> service, int id, boolean newStatus) {
        Class<T> entityClass = service.getEntityClass();
        Field[] declaredFields = entityClass.getDeclaredFields();
        UpdateWrapper<T> updateWrapper = new UpdateWrapper<>();
        boolean has = false;
        for (Field field : declaredFields) {
            field.setAccessible(true);
            String fieldName = field.getName();
            Status declaredAnnotation = field.getDeclaredAnnotation(Status.class);
            if (declaredAnnotation != null) {
                TableField tableField = field.getDeclaredAnnotation(TableField.class);
                String name;
                if (tableField == null) {
                    name = fieldName;
                } else {
                    name = tableField.value();
                }
                has = true;
                updateWrapper.set(name, newStatus);
            }
        }
        if (!has) {
            log.warn("类{}未找到status注解", entityClass);
            return false;
        }
        updateWrapper.eq("id", id);
        return service.update(updateWrapper);
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Target({ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    public @interface Updatable {
    }

    @Retention(RetentionPolicy.RUNTIME)
    @Target({ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    public @interface Status {
    }
}
